# -*- coding: utf-8 -*-
#
#       backuper.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import tarfile
import time
import os
import tempfile

"""
Class backuper handling backups
@author: Adam Schubert
@param backupTar: name of created/traget backup tar
"""


class Backuper:
    tar = None
    max_backup_size = 20971520  # 20MiB maximal archive size till cleanup
    backup_clean_keep = 20  # How many backup files keep in archive after cleanup

    def __init__(self, backup_tar, suffix='-backup.tar'):
        tar_name = backup_tar + suffix

        # Dropig old backups form file
        if os.path.isfile(tar_name):
            if os.path.getsize(tar_name) > self.max_backup_size:

                # create temp dir
                temp_dir_path = tempfile.mkdtemp()

                # extract
                t_file = tarfile.open(tar_name, 'r')
                t_file.extractall(temp_dir_path)
                t_file.close()

                # pack again
                n_file = tarfile.open(tar_name, 'w')

                # list all extracted files
                os.chdir(temp_dir_path)
                files = filter(os.path.isfile, os.listdir(temp_dir_path))
                files = [os.path.join(temp_dir_path, f) for f in files]
                files.sort(key=lambda x: os.path.getmtime(x))

                for file in files[-self.backup_clean_keep:]:  # last 10 files
                    n_file.add(file, arcname=os.path.basename(file))

                    n_file.close()

        self.tar = tarfile.TarFile(tar_name, 'a')

    """
    Adds backup to opened backup tar
    @param backupName: name of created file in tar
    @param data: string to store in file
    """

    def add_backup(self, file):
        # add current date in format Y-m-d H:i:s
        backup_name = time.strftime("%Y-%m-%d %H:%M:%S") + ' ' + os.path.basename(file)

        self.tar.add(file, arcname=backup_name)

    """
    Destructor, handles tar end
    """

    def __del__(self):
        if self.tar:
            self.tar.close()
