# -*- coding: utf-8 -*-
#
#       ftpd.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import module
import shadowd
import passwd
import group
import os

"""
Class ftpd handling ftpd configuration manipulation
@author: Adam Schubert
@param config: Configuration array
@param handler: Function handling Work statuses
"""


class ftpd(module.module):
    shadowd = None
    passwd = None
    group = None
    ftpRoot = None

    def __init__(self, config, handler=None):
        module.module.__init__(self, config, handler)
        self.ftpRoot = self.config['ftp_root']

        if not os.path.isdir(self.ftpRoot):
            raise Exception('FTP root {} is not a directory or dont exists!'.format(self.ftpRoot))

        # initiate all classes we need
        self.shadowd = shadowd.shadowd()
        self.passwd = passwd.passwd()
        self.group = group.group()

    """
    Changes FTP password
    @param username: Username to change password
    @param password: New password to set
    """

    def changePassword(self, username, password):
        if self.passwd.user_exists(username) and self.shadowd.user_exists(username):
            self.shadowd.changePassword(username, password)
            self.shadowd.save()
        else:
            raise Exception('User {} not found in system users!'.format(username))

    """
    Changes users name
    @param username: old username to change
    @param newUsername: new username to change to
    """

    def changeUsername(self, group, username, newUsername):
        if self.passwd.user_exists(username) and self.shadowd.user_exists(username):
            self.passwd.changeUsername(username, newUsername)
            self.shadowd.changeUsername(username, newUsername)
            self.group.delUser(group, username)
            self.group.addUser(group, newUsername)
            self.passwd.save()
            self.shadowd.save()
            self.group.save()
        else:
            raise Exception('User {} not found in system users!'.format(username))

    """
    Changes usernames homedir path
    @param username: username of user to change path
    @param path: new path to set
    """

    def changePath(self, username, path):
        ftp_path = os.path.join(self.ftpRoot, path)
        if self.passwd.user_exists(username):
            self.passwd.changeHome(username, ftp_path)
            self.passwd.save()
        else:
            raise Exception('User {} not found in system users!'.format(username))

    """
    Creates new FTP acces
    @param username: Username to create
    @param password: Password to set
    @param path: Target path of FTP
    """

    def addFtp(self, username, group, password, path):

        ftp_path = os.path.join(self.ftpRoot, path)

        if not self.passwd.user_exists(username) and not self.shadowd.user_exists(username):
            if self.group.group_exists(group):
                g_id = self.group.getGroupId(group)
            else:
                g_id = self.group.addGroup(group)
            self.passwd.addUser(username, g_id, ftp_path)
            self.shadowd.addUser(username, password)
            self.group.addUser(group, username)

            self.passwd.save()
            self.shadowd.save()
            self.group.save()
        else:
            raise Exception('User {} already exists in system users!'.format(username))

    """
    Removes FTP acces
    @param username: Username to delete
    """

    def delFtp(self, username):
        if self.passwd.user_exists(username):
            self.passwd.delUser(username)
            self.passwd.save()
        if self.shadowd.user_exists(username):
            self.shadowd.delUser(username)
            self.shadowd.save()
