# -*- coding: utf-8 -*-
#
#       group.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import collections
import backuper

"""
Class passwd handling passwd config file manipulation
@author: Adam Schubert
"""


class group:
    group_path = '/etc/group'  # This is UNIX standard, no need to put it in config

    openedFile = {}
    oldFile = ""

    def __init__(self):
        try:
            with open(self.group_path, 'r') as f:
                for line in f:
                    self.oldFile += line
                    # tohle pri malformad passwd spadne na hubu samo
                    # groupname:password:100:adam,sadam
                    group_name, password, g_id, users = line.split(':')
                    self.openedFile.update({group_name.strip(): {'password': password.strip(), 'gId': int(g_id.strip()),
                                                                'users': users.strip().split(',')}})
                self.sortDataStorage()
        except IOError:
            raise Exception('Group file {} not found or its not allowed for reading!'.format(self.group_path))

    """
    Internal function for data sorting
    """

    def sortDataStorage(self):
        self.openedFile = collections.OrderedDict(sorted(self.openedFile.items()))

    """
    Return new group ID for newly created group
    """

    def getNewGroupId(self):
        biggiest = 0
        for row in self.openedFile:
            if self.openedFile[row]['gId'] > biggiest:
                biggiest = self.openedFile[row]['gId']

        return biggiest + 1

    """
    Adds group to group file
    @param groupname: groupname of new group
    @param password: password
    """

    def addGroup(self, groupname, password='x'):
        # check groupname
        if groupname in self.openedFile:
            raise Exception('Groupname {} already exists!'.format(groupname))

        g_id = self.getNewGroupId()
        # Add new group
        self.openedFile.update({groupname: {'password': password, 'gId': g_id, 'users': []}})
        self.sortDataStorage()
        return g_id

    """
    Removes group from group file
    @param username: username of removed user
    """

    def delGroup(self, groupname):
        if groupname in self.openedFile:
            del self.openedFile[groupname]
        else:
            raise Exception('Groupname {} not found!'.format(groupname))

    """
    Changes group name
    @param groupname: old groupname to change
    @param newGroupname: new groupname to change to
    """

    def changeGroupname(self, groupname, newGroupname):
        try:
            self.openedFile[newGroupname] = self.openedFile.pop(groupname)
        except KeyError:
            raise Exception('Group {}, not found ?'.format(groupname))

    """
    Saves passwd file after modifications are made
    """

    def save(self):
        file_content = ""
        for row in self.openedFile:
            data = self.openedFile[row]
            file_content += ':'.join([row, data['password'], str(data['gId']), ','.join(data['users']) + "\n"])

        # create backup before we try to owerwrite it
        backup = backuper.Backuper(self.group_path)
        backup.add_backup(self.group_path)

        try:
            with open(self.group_path, 'w') as groupFile:
                groupFile.write(file_content)
        except IOError:
            raise Exception('Cannot write to group file {}, is writtable ?'.format(self.group_path))

    """
    Checks if user exists in passwd file
    @param username: Username to check
    """

    def group_exists(self, groupname):
        if groupname in self.openedFile:
            return True
        else:
            return False

    """
    Adds user into group
    @param groupname: groupname to add user
    @param username: username to add into group
    """

    def addUser(self, groupname, username):
        if not self.group_exists(groupname):
            raise Exception('Group {} not found!'.format(groupname))

        if username in self.openedFile[groupname]['users']:
            raise Exception('User {} is alreadyin group {}'.format(username, groupname))

        self.openedFile[groupname]['users'].append(username)

    """
    Removes user from group
    @param groupname: groupname from where user will be deleted
    @param username: username to delete
    """

    def delUser(self, groupname, username):
        if not self.group_exists(groupname):
            raise Exception('Group {} not found!'.format(groupname))

        if username not in self.openedFile[groupname]['users']:
            raise Exception('User {} not is group {}'.format(username, groupname))

        self.openedFile[groupname]['users'].remove(username)

    def getGroupId(self, groupname):
        if not self.group_exists(groupname):
            raise Exception('Group {} not found!'.format(groupname))

        return self.openedFile[groupname]['gId']
