# -*- coding: utf-8 -*-
#
#       maild.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import module
import os
import MySQLdb
import subprocess
import backuper
import pwd
import grp

"""
Class maild handling mail configuration manipulation
@author: Adam Schubert
@param config: Configuration array
@param handler: Function handling Work statuses
"""


class maild(module.module):
    config = None

    def __init__(self, config, handler):
        module.module.__init__(self, config, handler)
        self.config = config

    def get_db_connection(self):
        try:
            return MySQLdb.connect(self.config['host'], self.config['username'], self.config['password'],
                                   self.config['database'])
        except Exception as e:
            raise Exception('Failed to connect to {} error {}'.format(self.config['host'], str(e)))

    """
    Creates new email Domain
    @param domain: Domain name to create
    """

    def addDomain(self, domain):
        connection = self.get_db_connection()
        domain = domain.lower()
        binary_path = os.path.join(self.config['vpopmail_bin'], 'vadddomain')

        if not os.path.isfile(binary_path):
            raise Exception('Vpopmail binary {} not found!'.format(binary_path))

        subprocess.call([binary_path, domain, '-r'])

        self.setDomainTrash(domain)

        cursor = connection.cursor()
        # adds postmaster email to roundcube
        try:
            email = 'postmaster@' + domain
            command = "INSERT INTO `users` (`username`, `mail_host`, `language`, `created`, `last_login`) VALUES ('{}', '{}', '{}', NOW(), NOW());".format(
                email, self.config['mail_host'], self.config['language'])
            cursor.execute(command)
            userId = cursor.lastrowid

            command = "INSERT INTO `identities` (`user_id`, `changed`, `standard`, `email`, `name`) VALUES ('{}', NOW(), 1, '{}', '');".format(
                userId, email)
            cursor.execute(command)
            connection.commit()
            connection.close()
        except Exception as e:
            connection.rollback()
            connection.close()
            raise e

    def domain_exists(self, domain):
        domain = domain.lower()
        binary_path = os.path.join(self.config['vpopmail_bin'], 'vdominfo')
        read = subprocess.Popen([binary_path, '-d', domain], stdout=subprocess.PIPE).communicate()[0]
        if 'Invalid' in read:
            return False
        else:
            return read.splitlines()[0]

    """
    Deletes email Domain
    @param domain: Domain name to delete
    """

    def delDomain(self, domain):
        connection = self.get_db_connection()
        domain = domain.lower()
        binary_path = os.path.join(self.config['vpopmail_bin'], 'vdeldomain')

        if not os.path.isfile(binary_path):
            raise Exception('Vpopmail binary {} not found!'.format(binary_path))

        # backup whole domain
        domain_exists = self.domain_exists(domain)
        if domain_exists:
            backup_target = os.path.join(self.config['backup_path'], domain)
            backup = backuper.Backuper(backup_target, '.tar')
            backup.add_backup(domain_exists)

        subprocess.call([binary_path, domain])
        cursor = connection.cursor()

        try:
            email = 'postmaster@' + domain
            command = "DELETE FROM `users` WHERE `username` = '{}';".format(email)
            cursor.execute(command)
            connection.commit()
            connection.close()
        except Exception as e:
            connection.rollback()
            connection.close()
            raise e

    """
    Creates new user to existing domain
    @param email: email with domain, eq: example@example.com
    """

    def addUser(self, email, password):
        connection = self.get_db_connection()
        email = email.lower()
        binary_path = os.path.join(self.config['vpopmail_bin'], 'vadduser')

        if not os.path.isfile(binary_path):
            raise Exception('Vpopmail binary {} not found!'.format(binary_path))

        if not self.validateEmail(email):
            raise Exception('Email {} is not valid!'.format(email))

        name, domain = email.split('@')

        # Create users domain if dont exists, thanks this we dont need call addDomain from remote at all

        if not self.domain_exists(domain):
            self.addDomain(domain)

        cursor = connection.cursor()

        try:
            # user exist ?
            command = "SELECT COUNT(1) FROM `users` WHERE `username` = '{}'".format(email)
            cursor.execute(command)
            if cursor.fetchone()[0]:
                raise Exception('User {} exists in users table'.format(email))

            command = "INSERT INTO `users` (`username`, `mail_host`, `language`, `created`, `last_login`) VALUES ('{}', '{}', '{}', NOW(), NOW());".format(
                email, self.config['mail_host'], self.config['language'])
            cursor.execute(command)
            user_id = cursor.lastrowid

            command = "INSERT INTO `identities` (`user_id`, `changed`, `standard`, `email`, `name`) VALUES ('{}', NOW(), 1, '{}', '');".format(
                user_id, email)
            cursor.execute(command)
            connection.commit()

            # Call binary after user is added, this can fail on duplicity, but we dont care...
            # Error: User already exists  -E
            # Random password: postmaster -OK
            subprocess.call([binary_path, email, password])

            exist = self.domain_exists(domain)

            if exist:

                # Create .qmail-name
                q_mail_path = os.path.join(exist, '.qmail-{}'.format(name.replace('.', ':')))
                try:
                    with open(q_mail_path, 'w') as qfile:
                        qfile.write(self.config['domain_qmail_content'])
                except IOError:
                    raise Exception('Cannot write to file {}, is writable ?'.format(q_mail_path))

                # Create default sieve rule
                sieve_dir_path = os.path.join(exist, name, '.sieve')

                sieve_uid = pwd.getpwnam(self.config['vpopmail_user']).pw_uid
                sieve_gid = grp.getgrnam(self.config['vpopmail_group']).gr_gid

                if not os.path.isdir(sieve_dir_path):
                    os.mkdir(sieve_dir_path, 0o700)
                    os.chown(sieve_dir_path, sieve_uid, sieve_gid)

                sieve_file_path = os.path.join(sieve_dir_path, 'roundcube.sieve')
                link_file_path = os.path.join(sieve_dir_path, 'dovecot.sieve')
                sieve_content = []
                sieve_content.append('require ["fileinto"];')
                sieve_content.append('# rule:[spam]')
                sieve_content.append('if header :contains "subject" "[SPAM]"')
                sieve_content.append('{')
                sieve_content.append('fileinto "Junk";')
                sieve_content.append('}')

                try:
                    with open(sieve_file_path, 'w') as sieve_file:
                        sieve_file.write('\n'.join(sieve_content))

                    # Set user and group + mod
                    os.chmod(sieve_file_path, 0o600)
                    os.chown(sieve_file_path, sieve_uid, sieve_gid)

                    os.symlink(sieve_file_path, link_file_path)
                    os.lchown(link_file_path, sieve_uid, sieve_gid)
                except IOError:
                    raise Exception('Cannot write to file {}, is writable ?'.format(sieve_file_path))

            connection.close()
        except Exception as e:
            connection.rollback()
            connection.close()
            raise e

    """
    Deletes user
    @param email: email with domain, eq: example@example.com
    """

    def delUser(self, email):
        connection = self.get_db_connection()
        email = email.lower()
        binary_path = os.path.join(self.config['vpopmail_bin'], 'vdeluser')

        if not os.path.isfile(binary_path):
            raise Exception('Vpopmail binary {} not found!'.format(binary_path))

        if not self.validateEmail(email):
            raise Exception('Email {} is not valid!'.format(email))

        name, domain = email.split('@')

        # Backup user data
        domain_exist = self.domain_exists(domain)
        if domain_exist:
            backup_source = os.path.join(domain_exist, name)
            backup_target = os.path.join(self.config['backup_path'], '{}-{}'.format(domain, name))
            backup = backuper.Backuper(backup_target, '.tar')
            backup.add_backup(backup_source)

        subprocess.call([binary_path, email])

        if domain_exist:
            q_mail_path = os.path.join(domain_exist, '.qmail-{}'.format(name.replace('.', ':')))
            if os.path.isfile(q_mail_path):
                os.remove(q_mail_path)
        cursor = connection.cursor()

        try:
            command = "DELETE FROM `users` WHERE `username` = '{}';".format(email)
            cursor.execute(command)
            connection.commit()
            connection.close()
        except Exception as e:
            connection.rollback()
            connection.close()
            raise e

    """
    Creates new Alias
    @param fromEmail: original existing email (or alias)
    @param toEmail: target alias
    """

    def addAlias(self, fromEmail, toEmail):
        connection = self.get_db_connection()
        fromEmail = fromEmail.lower()
        toEmail = toEmail.lower()
        binary_path = os.path.join(self.config['vpopmail_bin'], 'valias')
        if not os.path.isfile(binary_path):
            raise Exception('Vpopmail binary {} not found!'.format(binary_path))
        if not self.validateEmail(fromEmail):
            raise Exception('Email {} is not valid!'.format(fromEmail))
        if not self.validateEmail(toEmail):
            raise Exception('Email {} is not valid!'.format(toEmail))

        subprocess.call([binary_path, '-i', fromEmail, toEmail])

        cursor = connection.cursor()

        try:
            command = "SELECT `user_id` FROM `users` WHERE `username` = '{}'".format(fromEmail)
            cursor.execute(command)

            res = cursor.fetchone()
            if res:
                userId = res[0]
            else:
                raise Exception('User {} not found in users table!'.format(fromEmail))

            # Alias exist ?
            command = "SELECT COUNT(1) FROM `identities` WHERE `email` = '{}' AND `user_id` = {}".format(toEmail,
                                                                                                         userId)
            cursor.execute(command)
            if cursor.fetchone()[0] == 0:
                command = "INSERT INTO `identities` (`user_id`, `changed`, `standard`, `email`, `name`) VALUES ('{}', NOW(), 0, '{}', '{}');".format(
                    userId, toEmail, toEmail)
                cursor.execute(command)
                connection.commit()
                connection.close()
        except Exception as e:
            connection.rollback()
            connection.close()
            raise e

    """
    Deletes alias
    @param fromEmail: original existing email (or alias)
    @param toEmail: target alias
    """

    def delAlias(self, toEmail):
        connection = self.get_db_connection()
        toEmail = toEmail.lower()
        binary_path = os.path.join(self.config['vpopmail_bin'], 'valias')

        if not os.path.isfile(binary_path):
            raise Exception('Vpopmail binary {} not found!'.format(binary_path))
        if not self.validateEmail(toEmail):
            raise Exception('Email {} is not valid!'.format(toEmail))

        subprocess.call([binary_path, '-d', toEmail])
        cursor = connection.cursor()
        try:
            command = "DELETE FROM `identities` WHERE `email` = '{}' AND `standard` = 0;".format(toEmail)
            cursor.execute(command)
            connection.commit()
            connection.close()
        except Exception as e:
            connection.rollback()
            connection.close()
            raise e

    """
    Enables or disables domain trash on domain
    @parma domain: domain for domain Trash
    @param target: target email (trash), if is None, domain trash is disabled
    """

    def setDomainTrash(self, domain, target=None):
        domain = domain.lower()
        exist = self.domain_exists(domain)
        if exist:
            filepath = os.path.join(exist, '.qmail-default')
            if target:
                target = target.lower()
                content = self.config['domain_qmail_content_trash'].format(target)
            else:
                content = self.config['domain_qmail_content']

            try:
                with open(filepath, 'w') as qfile:
                    qfile.write(content)
            except IOError:
                raise Exception('Cannot write to file {}, is writtable ?'.format(filepath))

    def changePassword(self, email, password):
        binary_path = os.path.join(self.config['vpopmail_bin'], 'vpasswd')

        if not os.path.isfile(binary_path):
            raise Exception('Vpopmail binary {} not found!'.format(binary_path))

        subprocess.call([binary_path, email, password])
