# -*- coding: utf-8 -*-
#
#       module.py
#
#       Copyright 2015 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import threading
import subprocess
import time
import re
import unicodedata

"""
Class module is base for other modules
@author: Adam Schubert
@param config: config array for child module
@param handler: handler called after end of thread
"""


class module:
    workList = []
    config = None
    handler = None

    def __init__(self, config, handler=None):
        self.thread = threading.Thread(target=self.run)
        # threading.Thread.__init__(self)

        self.handler = handler
        self.config = config

    """
    Adds work to thread and start work if is not running
    @param work: work command
    """

    def addWork(self, work):
        self.workList.append(work)

        # start work if is stopped
        if not self.thread.isAlive():
            self.thread = threading.Thread(target=self.run)
            self.thread.start()

    """
    This is part of threading.Thread, it started by self.start()
    Runs working Thread
    """

    def run(self):
        while len(self.workList) > 0:
            work = self.workList.pop()
            # run a function
            try:
                getattr(self, work['action'])(*work['params'])
                status = True
                message = 'Work finished'
            except Exception as ex:
                status = False
                message = str(ex)
            if self.handler:
                self.handler(work, status, message)

        # When we get here it means our thread ended, so we can reload service
        self.callService('reload')

    """
    Calls service if any to reload/restart etc new config
    @param command: command to call
    """

    def callService(self, command):
        if 'service_command' in self.config:
            parts = self.config['service_command'].split(' ')
            commands = parts[-1].split('|')
            del parts[-1]

            if command in commands:
                parts.append(command)
                print('Running command {}'.format(command))
                try:
                    subprocess.call(parts)
                except Exception:
                    pass
            elif command == 'reload' and 'restart' in commands:
                print('{} is not supported, but restart found, gonna try it...'.format(command))

                # stop
                print('Running command {}'.format('restart'))
                parts.append('restart')
                try:
                    subprocess.call(parts)
                except Exception:
                    pass
            else:
                print('{} is not supported'.format(command))

    # helper functions

    """
    Checks if email is valid
    @param email: email to check
    """

    def validateEmail(self, email):
        if not re.match(r"[^@]+@[^@]+\.[^@]+", email):
            return False
        else:
            return True

    def clearString(self, string):
        return unicodedata.normalize('NFKD', string).encode('ascii', 'ignore')
