# -*- coding: utf-8 -*-
#
#       spamassassind.py
#
#       Copyright 2015 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import module
import os
import re

"""
Class spamassassind handling spamassassin configuration manipulation
@author: Adam Schubert
@param config: Configuration array
@param handler: Function handling Work statuses
"""


class spamassassind(module.module):
    config = None

    def __init__(self, config, handler):
        module.module.__init__(self, config, handler)
        self.config = config

        self.blacklist_action = 'blacklist_from'
        self.whitelist_action = 'whitelist_from'
        self.bw = self.parseBwFile(self.config['bwfile'])

    def parseBwFile(self, path):
        if os.path.isfile(path) is False:
            raise Exception('Bw file {} not found'.format(path))

        with open(path) as f:
            content = f.readlines()

        if content is None:
            raise Exception('Failed to open Bw file {}'.format(path))

        bwinfo = {'blacklisted': [], 'whitelisted': []}
        for line in content:
            striped = line.strip()
            if len(striped):
                splited = striped.split(' ')
                if len(splited) != 2:
                    raise Exception('File {} is malformed'.format(path))

                action, email = splited
                if action == self.blacklist_action:
                    bwinfo['blacklisted'].append(email)
                elif action == self.whitelist_action:
                    bwinfo['whitelisted'].append(email)
                else:
                    raise Exception('Unknow action {}'.format(action))

        return bwinfo

    def saveBwFile(self):
        content = []
        for blacklisted in self.bw['blacklisted']:
            content.append(self.blacklist_action + ' ' + blacklisted)

        for whitelisted in self.bw['whitelisted']:
            content.append(self.whitelist_action + ' ' + whitelisted)

        with open(self.config['bwfile'], 'w') as file:
            file.write("\n".join(content))

    """
    Adds new spamassasin bw rule
    @param domain: Domain name to create
    """

    def addRule(self, email, action='blacklist_from'):

        if email in self.bw['blacklisted'] or email in self.bw['whitelisted']:
            raise Exception('Email {} already exists in bwfile'.format(email))

        if action == self.blacklist_action:
            self.bw['blacklisted'].append(email)
        elif action == self.whitelist_action:
            self.bw['whitelisted'].append(email)
        else:
            raise Exception('Unknow action {}'.format(action))
        self.saveBwFile()

    def deleteRule(self, email):
        if email not in self.bw['blacklisted'] and email not in self.bw['whitelisted']:
            raise Exception('Email {} not found in bw file'.format(email))

        if email in self.bw['blacklisted']:
            self.bw['blacklisted'].remove(email)

        if email in self.bw['whitelisted']:
            self.bw['whitelisted'].remove(email)

        self.saveBwFile()

    """
    Deletes email Domain
    @param domain: Domain name to delete
    """

    def setSpamScore(self, score):
        path = self.config['localfile']
        if os.path.isfile(path) is False:
            raise Exception('local file {} not found'.format(path))
        with open(path) as f:
            content = f.read()
            content = re.sub(r"required_score\s+[0-9]+\.[0-9]+", "required_score {}".format(float(score)), content)

            with open(path, 'w') as file:
                file.write(content)
